﻿/* INFO
 * 
 * Author: PaT
 * Website: www.k-cheat.de
 * 23 June 2011
 * 
 */
using System;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;

namespace KButton
{
    [Description("Knuddels Button Control")]
    [ToolboxBitmap(typeof (Button))]
    public sealed partial class KButton : Control
    {
        #region " FELDER "

        #region " DEFAULT "

        private readonly Font _DefaultFont = new Font("Arial", 11.0F, FontStyle.Bold);
        private readonly Color _DefaultForeColor = Color.White;
        private readonly Size _DefaultSize = new Size(175, 28);

        #endregion

        private int _Alpha1 = 200;
        private int _Alpha2 = 50;
        private int _AlphaMouseOver = 50;
        private Color _BorderColor = Color.Black;
        private ButtonState _ButtonState = ButtonState.Normal;
        private Corners _Corners = Corners.All;
        private Color _GlassColor = Color.White;
        private Color _GradientBackColor = Color.FromArgb(38, 55, 185);

        private string _Text;

        #endregion

        #region " EIGENSCHAFTEN "

        //+ -------------------------
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        public override string Text
        {
            get { return _Text; }
            set
            {
                _Text = value;
                Invalidate();
            }
        }

        //+ -------------------------
        [DefaultValue(typeof (Color), "White")]
        public override Color ForeColor
        {
            get { return base.ForeColor; }
            set
            {
                base.ForeColor = value;
                Invalidate();
            }
        }

        //+ -------------------------
        [DefaultValue(typeof (Color), "Transparent")]
        public override Color BackColor
        {
            get { return base.BackColor; }
            set
            {
                base.BackColor = value;
                Invalidate();
            }
        }

        //+ -------------------------
        [DefaultValue(typeof(Font), "Arial, 11pt, style=Bold")]
        public override Font Font
        {
            get { return base.Font; }
            set
            {
                base.Font = value;
                Invalidate();
            }
        }

        //+ -------------------------
        protected override Size DefaultSize
        {
            get { return _DefaultSize; }
        }

        #region " AUSBLENDEN "

        [Browsable(false)]
        [EditorBrowsable(EditorBrowsableState.Never)]
        public new RightToLeft RightToLeft { get; set; }

        [Browsable(false)]
        [EditorBrowsable(EditorBrowsableState.Never)]
        public override Image BackgroundImage
        {
            get { return base.BackgroundImage; }
            set { base.BackgroundImage = value; }
        }

        [Browsable(false)]
        [EditorBrowsable(EditorBrowsableState.Never)]
        public override ImageLayout BackgroundImageLayout
        {
            get { return base.BackgroundImageLayout; }
            set { base.BackgroundImageLayout = value; }
        }

        #endregion

        #region " STYLE "

        [Category("Knuddels")]
        [Description("Gibt die Farbe vom Rand des Buttons an.")]
        public Color BorderColor
        {
            get { return _BorderColor; }
            set
            {
                _BorderColor = value;
                Invalidate();
            }
        }

        [Category("Knuddels")]
        [Description("Die Hintergrundfarbe des Buttons.")]
        public Color GradientBackColor
        {
            get { return _GradientBackColor; }
            set
            {
                _GradientBackColor = value;
                Invalidate();
            }
        }

        [Category("Knuddels")]
        [Description("Die Farbe des Glasseffektes.")]
        public Color GlassColor
        {
            get { return _GlassColor; }
            set
            {
                _GlassColor = value;
                Invalidate();
            }
        }

        [Category("Knuddels")]
        [DefaultValue(200)]
        public int Alpha1
        {
            get { return _Alpha1; }
            set
            {
                _Alpha1 = value;
                Invalidate();
            }
        }

        [Category("Knuddels")]
        [DefaultValue(50)]
        public int Alpha2
        {
            get { return _Alpha2; }
            set
            {
                _Alpha2 = value;
                Invalidate();
            }
        }

        [Category("Knuddels")]
        [DefaultValue(50)]
        public int AlphaMouseOver
        {
            get { return _AlphaMouseOver; }
            set { _AlphaMouseOver = value; }
        }

        [Category("Knuddels")]
        [DefaultValue(true)]
        public bool CornerRound
        {
            get { return _Corners == Corners.All ? true : false; }
            set
            {
                _Corners = value ? Corners.All : Corners.None;
                Invalidate();
            }
        }

        #endregion
        //+ -------------------------

        #endregion

        #region " KONSTRUKTOR "

        public KButton()
        {
            InitializeComponent();
            // -------------------
            SetStyle(
                ControlStyles.StandardClick
                | ControlStyles.SupportsTransparentBackColor
                | ControlStyles.UserPaint
                | ControlStyles.ResizeRedraw
                | ControlStyles.DoubleBuffer
                , true);
            // -------------------
            Font = _DefaultFont;
            ForeColor = _DefaultForeColor;
            Size = _DefaultSize;
        }

        #endregion

        #region " OVERRIDE "

        #region " MOUSE "

        protected override void OnMouseEnter(EventArgs e)
        {
            base.OnMouseEnter(e);
            _ButtonState = ButtonState.MouseOver;
            Invalidate();
        }

        protected override void OnMouseLeave(EventArgs e)
        {
            base.OnMouseLeave(e);
            _ButtonState = ButtonState.Normal;
            Invalidate();
        }

        protected override void OnMouseDown(MouseEventArgs e)
        {
            base.OnMouseDown(e);
            _ButtonState = ButtonState.Pressed;
            Invalidate();
        }

        protected override void OnMouseUp(MouseEventArgs e)
        {
            base.OnMouseDown(e);
            _ButtonState = ButtonState.MouseOver;
            Invalidate();
        }

        #endregion

        #endregion

        #region " PAINT "

        #region " ON | PAINT "

        protected override void OnPaint(PaintEventArgs e)
        {
            PaintBackground(e);
            PaintText(e);
        }

        #endregion

        #region " PAINT | TEXT "

        /// <summary>
        ///   Zeichnet den Text.
        /// </summary>
        /// <param name = "e"></param>
        private void PaintText(PaintEventArgs e)
        {
            Graphics g = e.Graphics;
            // ---------------------------------------------------
            //- Variablen deklarieren
            SolidBrush foreColorBrush = new SolidBrush(ForeColor);
            SizeF textSize = e.Graphics.MeasureString(Text, Font);
            int textX = (Size.Width/2) - (int) (textSize.Width/2);
            int textY = (Size.Height/2) - (int) (textSize.Height/2);
            // ---------------------------------------------------
            //- Mauszeiger reagieren..
            switch (_ButtonState)
            {
                case ButtonState.Pressed:
                    g.DrawString(Text, Font, foreColorBrush, textX + 1, textY + 1);
                    break;
                default:
                    g.DrawString(Text, Font, foreColorBrush, textX, textY);
                    break;
            }
            // ---------------------------------------------------
            //- Dispose
            foreColorBrush.Dispose();
        }

        #endregion

        #region " PAINT | BACKGROUND "

        /// <summary>
        ///   Zeichnet den Hintergrund.
        /// </summary>
        /// <param name = "e"></param>
        private void PaintBackground(PaintEventArgs e)
        {
            Graphics g = e.Graphics;
            Rectangle rect = ClientRectangle;
            g.SmoothingMode = SmoothingMode.Default;
            // ---------------------------------------------------
            //- Variablen deklarieren
            Pen borderPen = new Pen(BorderColor);
            Pen glassPen = null;

            SolidBrush gradientBackBrush = new SolidBrush(GradientBackColor);
            SolidBrush overlaySolidBrush = null;
            LinearGradientBrush spiegelBrush = null;

            GraphicsPath fullGlassPath = null;
            GraphicsPath spiegelPath = null;
            GraphicsPath spiegelPathOther = null;
            LinearGradientBrush spiegelBrushOther = null;
            // ---------------------------------------------------
            //- Rand zeichnen
            g.DrawPath(borderPen, CreateBorderPath(0, 0, rect.Width - 1, rect.Height - 1, 2, _Corners));
            // ---------------------------------------------------
            //- Hintergrundfarbe zeichnen
            GraphicsPath fillPath = CreateInnerPath(1, 1, rect.Width - 2, rect.Height - 2, 1, _Corners);
            g.FillPath(gradientBackBrush, fillPath);
            // ---------------------------------------------------
            //- Variablen instanzieieren
            Corners topCorners = CornerRound ? Corners.TopLeft | Corners.TopRight : Corners.None;
            spiegelPath = CreateInnerPath(1, 1, rect.Width - 2, (rect.Height - 2)/2, 1, topCorners);
            spiegelBrush = new LinearGradientBrush(new Rectangle(1, 1, Width - 2, Height / 2),
                                                       Color.FromArgb(Alpha1, GlassColor),
                                                       Color.FromArgb(Alpha2, GlassColor), LinearGradientMode.Vertical);
            // ---------------------------------------------------
            //- Mauszeiger reagieren..
            switch (_ButtonState)
            {
                case ButtonState.Normal: // Im Normalen Zustand.
                    glassPen = new Pen(Color.FromArgb(5, GlassColor));
                    g.FillPath(spiegelBrush, spiegelPath);
                    g.DrawLine(glassPen, 1, Height / 2, Width - 2, Height / 2);
                    break;
                case ButtonState.MouseOver: // Wenn sich die Maus über dem Button befindet.
                    g.FillPath(spiegelBrush, spiegelPath);
                    fullGlassPath = CreateInnerPath(1, 1, rect.Width - 1, rect.Height, 1);
                    overlaySolidBrush = new SolidBrush(Color.FromArgb(AlphaMouseOver, _GlassColor));
                    g.FillPath(overlaySolidBrush, fullGlassPath);
                    break;
                case ButtonState.Pressed: // Wenn auf dem Button geklickt wird.
                    // Spiegeleffekt umdrehen // Sieht zwar iwie dumm aus aber naja, Knuddelsstyle eben :/

                    Corners bottomCorners = CornerRound ? Corners.BottomLeft | Corners.BottomRight : Corners.None;

                    const int left = 1;
                    int top = rect.Height/2;
                    int width = rect.Width - 2;
                    int height = top - 1;
                    if (rect.Height%2 != 0)
                        height += 1;

                    spiegelPathOther = CreateInnerPath(left, top, width, height, 1, bottomCorners);

                    spiegelBrushOther = new LinearGradientBrush(new Rectangle(left, top - 1, width, height + 1),
                                                                    Color.FromArgb(Alpha2, GlassColor),
                                                                    Color.FromArgb(Alpha1, GlassColor),
                                                                    LinearGradientMode.Vertical);
                    // ----
                    g.FillPath(spiegelBrushOther, spiegelPathOther);
                    break;
            }
            // ---------------------------------------------------
            //- Dispose.
            borderPen.Dispose();
            if (glassPen != null) 
                glassPen.Dispose();

            gradientBackBrush.Dispose();
            spiegelBrush.Dispose();
            if (overlaySolidBrush != null) 
                overlaySolidBrush.Dispose();

            spiegelPath.Dispose();
            if (fullGlassPath != null) 
                fullGlassPath.Dispose();
            if (spiegelPathOther != null)
                spiegelPathOther.Dispose();
            if (spiegelBrushOther != null) 
                spiegelBrushOther.Dispose();
        }

        #endregion

        #endregion

        #region " HILFSMEMBER "

        #region " CREATEBORDERPATH | - Erstellt den Pfad für den Rand."

        private static GraphicsPath CreateBorderPath(int pX, int pY, int pWidth, int pHeight, int pRadius, Corners pCorners = Corners.All)
        {
            GraphicsPath graphicsPath = new GraphicsPath();
            graphicsPath.StartFigure();
            // --------------------------
            int widthLeft = pWidth + pX;
            int heightTop = pHeight + pY;
            //+ -------------------------------------
            //+ Rechts
            // - Oben
            if ((pCorners & Corners.TopRight) == Corners.TopRight)
                graphicsPath.AddLine(widthLeft - pRadius, pY, widthLeft, pY + pRadius);
            else
                graphicsPath.AddLines(new[] {new Point(widthLeft, pY), new Point(widthLeft, heightTop - pRadius)});
            // - Unten
            if ((pCorners & Corners.BottomRight) == Corners.BottomRight)
                graphicsPath.AddLine(widthLeft, heightTop - pRadius, widthLeft - pRadius, heightTop);
            else
                graphicsPath.AddLines(new[] {new Point(widthLeft, heightTop), new Point(widthLeft - pRadius, heightTop)});
            // ---------
            //+ Links
            // - Unten
            if ((pCorners & Corners.BottomLeft) == Corners.BottomLeft)
                graphicsPath.AddLine(pX + pRadius, heightTop, pX, heightTop - pRadius);
            else
                graphicsPath.AddLines(new[] {new Point(pX, heightTop), new Point(pX, pHeight - pRadius),});
            // - Oben
            if ((pCorners & Corners.TopLeft) == Corners.TopLeft)
                graphicsPath.AddLine(pX, pY + pRadius, pX + pRadius, pY);
            else
                graphicsPath.AddLines(new[] {new Point(pX, pY), new Point(pX + pRadius, pY)});
            //+ -------------------------------------         
            graphicsPath.CloseFigure();
            return graphicsPath;
        }

        #endregion

        #region " CREATEINNERPATH "

        private static GraphicsPath CreateInnerPath(int pX, int pY, int pWidth, int pHeight, int pRadius,
                                                    Corners pCorners = Corners.All)
        {
            GraphicsPath graphicsPath = new GraphicsPath();
            graphicsPath.StartFigure();
            // --------------------------
            int widthLeft = pWidth + pX;
            int heightTop = pHeight + pY;
            //+ -------------------------------------
            //+ Rechts
            // - Oben
            if ((pCorners & Corners.TopRight) == Corners.TopRight)
                graphicsPath.AddLine(widthLeft - pRadius, pY, widthLeft, pY + pRadius);
            else
                graphicsPath.AddLines(new[] {new Point(widthLeft, pY), new Point(widthLeft, heightTop - pRadius)});
            // - Unten
            if ((pCorners & Corners.BottomRight) == Corners.BottomRight)
            {
                graphicsPath.AddLine(widthLeft, heightTop - pRadius, widthLeft - 1, heightTop - pRadius);
                graphicsPath.AddLine(widthLeft - 1, heightTop - pRadius, widthLeft - 1, heightTop - pRadius + 1);
            }
            else
                graphicsPath.AddLines(new[] {new Point(widthLeft, heightTop), new Point(widthLeft - pRadius, heightTop)});
            // ---------
            //+ Links
            // - Unten
            if ((pCorners & Corners.BottomLeft) == Corners.BottomLeft)
            {
                graphicsPath.AddLine(pX + pRadius, heightTop, pX + pRadius, heightTop - 1);
                graphicsPath.AddLine(pX + pRadius, heightTop - 1, pX + pRadius - 1, heightTop - 1);
            }
            else
                graphicsPath.AddLines(new[] {new Point(pX, heightTop), new Point(pX, pHeight - pRadius),});
            // - Oben
            if ((pCorners & Corners.TopLeft) == Corners.TopLeft)
                graphicsPath.AddLine(pX, pY + pRadius, pX + pRadius, pY);
            else
                graphicsPath.AddLines(new[] {new Point(pX, pY), new Point(pX + pRadius, pY)});
            //+ -------------------------------------         
            graphicsPath.CloseFigure();
            return graphicsPath;
        }

        #endregion

        #region " ENUM | ECKEN "

        [Flags]
        private enum Corners
        {
            None = 0,
            TopLeft = 1,
            TopRight = 2,
            BottomLeft = 4,
            BottomRight = 8,
            All = TopLeft | TopRight | BottomLeft | BottomRight
        }

        #endregion

        #endregion

    }
}